#!/usr/bin/env python

import os

import pygame

import pyguts as spine

if __name__ == '__main__':
    pygame.init()

    width, height = (1024, 768)

    screen = pygame.display.set_mode((width, height))
    screen.fill((0,0,0))
    caption = 'PyGuts - A Pygame front-end based on the python-spine Runtime'
    pygame.display.set_caption(caption, 'Spine Runtime')

    atlasFile = os.path.realpath('./data/dragon.atlas')
    atlas = spine.Atlas(file=atlasFile)

    skeletonJson = spine.SkeletonJson(spine.AtlasAttachmentLoader(atlas))

    skeletonFile = os.path.realpath('./data/dragon-skeleton.json')
    skeletonData = skeletonJson.readSkeletonData(skeletonFile)

    animationFile = os.path.realpath('./data/dragon-flying.json')
    animation = skeletonJson.readAnimation(file=animationFile, 
                                           skeletonData=skeletonData)       

    skeleton = spine.Skeleton(skeletonData=skeletonData)
    skeleton.debug = True

    skeleton.setToBindPose()
    skeleton.x = 480
    skeleton.y = 320
    skeleton.flipX = False
    skeleton.flipY = False
    skeleton.updateWorldTransform()

    clock = pygame.time.Clock()    
    animationTime = 0.0

    done = False

    while not done:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                done = True
            elif event.type == pygame.KEYDOWN:
                if pygame.key.get_pressed()[pygame.K_ESCAPE]:
                    done = True
        clock.tick(0)
        animationTime += clock.get_time() / 1000.0
        animation.apply(skeleton=skeleton,
                        time=animationTime,
                        loop=True)
        skeleton.updateWorldTransform()
        screen.fill((0, 0, 0))
        skeleton.draw(screen, 0)
        pygame.display.set_caption('%s  %.2f' % (caption, clock.get_fps()), 'Spine Runtime')
        pygame.display.flip()
pygame.quit()



import spine

import pygame

class Color(object):
    def __init__(self):
        super(Color, self).__init__()
        self.r = 0.0
        self.g = 0.0
        self.b = 0.0
        self.a = 0.0

class TextureCoordinates(object):
    def __init__(self):
        super(TextureCoordinates, self).__init__()
        self.x = 0.0
        self.y = 0.0

class Vertex(object):
    def __init__(self):
        super(Vertex, self).__init__()
        self.color = Color()
        self.texCoords = TextureCoordinates()

class RegionAttachment(spine.RegionAttachment):
    def __init__(self, region):
        super(RegionAttachment, self).__init__()
        self.verticies = [Vertex(), Vertex(), Vertex(), Vertex()]
        self.u = region.x
        self.u2 = self.u + region.width
        self.v = region.y
        self.v2 = self.v + region.height
        self.rect = pygame.Rect((self.u, self.v, region.width, region.height))
        self.texture = region.page.texture.subsurface(self.rect)
        self.offset = pygame.Rect(0, 0, region.width, region.height)
        self.offset.center = (region.width / 2, region.height / 2)
        if region.rotate:
            self.verticies[1].texCoords.x = self.u
            self.verticies[1].texCoords.y = self.v2
            self.verticies[2].texCoords.x = self.u
            self.verticies[2].texCoords.y = self.v
            self.verticies[3].texCoords.x = self.u2
            self.verticies[3].texCoords.y = self.v
            self.verticies[0].texCoords.x = self.u2
            self.verticies[0].texCoords.y = self.v2
        else:
            self.verticies[0].texCoords.x = self.u
            self.verticies[0].texCoords.y = self.v2
            self.verticies[1].texCoords.x = self.u
            self.verticies[1].texCoords.y = self.v
            self.verticies[2].texCoords.x = self.u2
            self.verticies[2].texCoords.y = self.v
            self.verticies[3].texCoords.x = self.u2
            self.verticies[3].texCoords.y = self.v2

    
    def draw(self, slot):
        skeleton = slot.skeleton

        r = skeleton.r * slot.r * 255.0
        g = skeleton.g * slot.g * 255.0
        b = skeleton.b * slot.b * 255.0
        a = skeleton.a * slot.a * 255.0

        self.verticies[0].color.r = r
        self.verticies[0].color.g = g
        self.verticies[0].color.b = b
        self.verticies[0].color.a = a
        self.verticies[1].color.r = r
        self.verticies[1].color.g = g
        self.verticies[1].color.b = b
        self.verticies[1].color.a = a
        self.verticies[2].color.r = r
        self.verticies[2].color.g = g
        self.verticies[2].color.b = b
        self.verticies[2].color.a = a
        self.verticies[3].color.r = r
        self.verticies[3].color.g = g
        self.verticies[3].color.b = b
        self.verticies[3].color.a = a

        self.updateOffset()
        self.updateWorldVerticies(slot.bone)

        skeleton.texture = self.texture
        skeleton.vertexArray.append(self.verticies[0])
        skeleton.vertexArray.append(self.verticies[1])
        skeleton.vertexArray.append(self.verticies[2])
        skeleton.vertexArray.append(self.verticies[3])


    def updateWorldVertices(self, bone):
        x = bone.worldX
        y = bone.worldY
        m00 = bone.m00
        m01 = bone.m01
        m10 = bone.m10
        m11 = bone.m11
        self.verticies[0].position.x = self.offset[0] * m00 + offset[1] * m01 + x
        self.verticies[0].position.y = self.offset[0] * m10 + offset[1] * m11 + y
        self.verticies[1].position.x = self.offset[2] * m00 + offset[3] * m01 + x
        self.verticies[1].position.y = self.offset[2] * m10 + offset[3] * m11 + y
        self.verticies[2].position.x = self.offset[4] * m00 + offset[5] * m01 + x
        self.verticies[2].position.y = self.offset[4] * m10 + offset[5] * m11 + y
        self.verticies[3].position.x = self.offset[6] * m00 + offset[7] * m01 + x
        self.verticies[3].position.y = self.offset[6] * m10 + offset[7] * m11 + y
        



import math

import spine

import pygame

class Circle(object):
    def __init__(self, x, y, r):
        super(Circle, self).__init__()
        self.x = x
        self.y = y
        self.r = r
        self.color = (0, 255, 0, 255)


class Line(object):
    def __init__(self, length):
        super(Line, self).__init__()
        self.x = 0.0
        self.y = 0.0
        self.x1 = 0.0
        self.x2 = 0.0
        self.length = length
        self.rotation = 0.0
        self.color = (255, 0, 0, 255)
        self.texture = pygame.Surface((640, 480), pygame.SRCALPHA, 32)
        pygame.draw.rect(self.texture, (255, 255, 0, 64), (0, 0, self.texture.get_width(), self.texture.get_height()), 1)


    def rotate(self):
        return pygame.transform.rotozoom(self.texture, self.rotation, self.xScale)


class Skeleton(spine.Skeleton):
    def __init__(self, skeletonData):
        super(Skeleton, self).__init__(skeletonData=skeletonData)
        self.x = 0
        self.y = 0
        self.texture = None
        self.debug = True
        self.images = []
        self.clock = None

    def draw(self, screen, states):
        x = 0
        y = 0

        for slot in self.drawOrder:
            if slot.attachment:
                texture = slot.attachment.texture.copy()
                if texture:
                    x = slot.bone.worldX + slot.attachment.x  * slot.bone.m00 + slot.attachment.y * slot.bone.m01
                    y = -(slot.bone.worldY + slot.attachment.x * slot.bone.m10 + slot.attachment.y * slot.bone.m11)
                    rotation = -(slot.bone.worldRotation + slot.attachment.rotation)
                    xScale = slot.bone.worldScaleX + slot.attachment.scaleX - 1
                    yScale = slot.bone.worldScaleY + slot.attachment.scaleY - 1

                    x += self.x
                    y += self.y

                    if self.flipX:
                        xScale = -xScale
                        rotation = -rotation
                    if self.flipY:
                        yScale = -yScale
                        rotation = -rotation

                    flipX = False
                    flipY = False

                    if xScale < 0:
                        flipX = True
                        xScale = math.fabs(xScale)
                    if yScale < 0:
                        flipY = True
                        yScale = math.fabs(yScale)

                    
                    texture.fill((slot.r, slot.g, slot.b, slot.a), None, pygame.BLEND_RGBA_MULT)

                    center = texture.get_rect().center                    
                    texture = pygame.transform.flip(texture, flipX, flipY)
                    texture = pygame.transform.smoothscale(texture, (int(texture.get_width() * xScale), int(texture.get_height() * yScale)))
                    texture = pygame.transform.rotozoom(texture, -rotation, 1)

                    # Center image
                    x = x - texture.get_width() / 2
                    y = y - texture.get_height() / 2
                    screen.blit(texture, (x, y))

        if self.debug:
            if not self.clock:
                self.clock = pygame.time.Clock()
            self.clock.tick()
            # Draw the FPS in the bottom right corner.
            pygame.font.init()
            myfont = pygame.font.SysFont(None, 24, bold=True)
            mytext = myfont.render('FPS: %.2f' % self.clock.get_fps(), True, (255, 255, 255))

            screen.blit(mytext, (screen.get_width() - mytext.get_width(), screen.get_height() - mytext.get_height()))

            for bone in self.bones:

                if not bone.line:
                    bone.line = Line(bone.data.length)
                bone.line.x = bone.worldX + self.x
                bone.line.y = -bone.worldY + self.y
                bone.line.rotation = -bone.worldRotation
                bone.line.color = (255, 0, 0)

                if self.flipX:
                    bone.line.xScale = -1
                    bone.line.rotation = -bone.line.rotation
                else:
                    bone.line.xScale = 1
                if self.flipY:
                    bone.line.yScale = -1
                    bone.line.rotation = -bone.line.rotation
                else:
                    bone.line.yScale = 1

                bone.line.x1 = bone.line.x + math.cos(math.radians(bone.line.rotation)) * bone.line.length
                bone.line.y1 = bone.line.y + math.sin(math.radians(bone.line.rotation)) * bone.line.length

                pygame.draw.line(screen, bone.line.color, (bone.line.x, bone.line.y), (bone.line.x1, bone.line.y1))

                if not bone.circle:
                    bone.circle = Circle(0, 0, 3)
                bone.circle.x = int(bone.worldX) + self.x
                bone.circle.y = -int(bone.worldY) + self.y
                bone.circle.color = (0, 255, 0)
                    
                if 'top left' in bone.data.name:
                    bone.circle.color = (255, 0, 0)
                if 'top right' in bone.data.name:
                    bone.circle.color = (255, 140, 0)
                if 'bottom right' in bone.data.name:
                    bone.circle.color = (255, 255, 0)
                if 'bottom left' in bone.data.name:
                    bone.circle.color = (199, 21, 133)

                pygame.draw.circle(screen, 
                                   bone.circle.color,
                                   (bone.circle.x, bone.circle.y),
                                   bone.circle.r,
                                   0)



#!/usr/bin/env python

from setuptools import setup

setup(name='pyguts',
      version='1.0b12',
      description='A pygame front-end for spine-python.',
      author='Terry Simons',
      author_email='terry.simons@gmail.com',
      url='https://github.com/terrysimons/spine-python/pyguts',
      package_dir={'pyguts': 'src'},
      packages=['pyguts'],
      classifiers=['License :: OSI Approved :: BSD License'],
      install_requires=['spine_python']
     )



import os


import pygame


import spine

class AtlasPage(spine.AtlasPage):
    def __init__(self):
        super(AtlasPage, self).__init__()
        self.texture = None


class AtlasRegion(spine.AtlasRegion):
    def __init__(self):
        super(AtlasRegion, self).__init__()
        self.page = None


class Atlas(spine.Atlas):
    def __init__(self, file):
        super(Atlas, self).__init__()
        super(Atlas, self).loadWithFile(file)


    def newAtlasPage(self, name):
        page = AtlasPage()
        page.texture = pygame.image.load(os.path.realpath(name)).convert_alpha()
        return page


    def newAtlasRegion(self, page):
        region = AtlasRegion()
        region.page = page
        return region


    def findRegion(self, name):
        return super(Atlas, self).findRegion(name)



import spine

import RegionAttachment

class AtlasAttachmentLoader(spine.AttachmentLoader.AttachmentLoader):
    def __init__(self, atlas):
        self.atlas = atlas
        
    def newAttachment(self, type, name):
        if type == spine.AttachmentLoader.AttachmentType.region:
            region = self.atlas.findRegion(name)
            if not region:
                raise Exception("Atlas region not found: %s" % name)
            return RegionAttachment.RegionAttachment(region)
        else:
            raise Exception('Unknown attachment type: %s' % type)

    



#!/usr/bin/env python

import os

import pygame

import pyguts as spine

if __name__ == '__main__':
    pygame.init()

    width, height = (640, 480)

    screen = pygame.display.set_mode((width, height))
    screen.fill((0,0,0))
    caption = 'PyGuts - A Pygame front-end based on the python-spine Runtime'
    pygame.display.set_caption(caption, 'Spine Runtime')

    atlasFile = os.path.realpath('./data/spineboy2.atlas')
    atlas = spine.Atlas(file=atlasFile)

    skeletonJson = spine.SkeletonJson(spine.AtlasAttachmentLoader(atlas))

    skeletonFile = os.path.realpath('./data/spineboy2-skeleton.json')
    skeletonData = skeletonJson.readSkeletonData(skeletonFile)

    animationFile = os.path.realpath('./data/spineboy2-walk.json')
    walkAnimation = skeletonJson.readAnimation(file=animationFile, 
                                               skeletonData=skeletonData)       

    animationFile = os.path.realpath('./data/spineboy2-jump.json')
    jumpAnimation = skeletonJson.readAnimation(file=animationFile, 
                                               skeletonData=skeletonData)       

    skeleton = spine.Skeleton(skeletonData=skeletonData)
    skeleton.debug = True

    skeleton.setToBindPose()
    skeleton.x = -50
    skeleton.y = 400
    skeleton.flipX = False
    skeleton.flipY = False
    skeleton.updateWorldTransform()    

    clock = pygame.time.Clock()    
    animationTime = 0.0

    done = False

    time = 0

    while not done:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                done = True
            elif event.type == pygame.KEYDOWN:
                if pygame.key.get_pressed()[pygame.K_ESCAPE]:
                    done = True
        clock.tick(60.0)

        delta = clock.get_time() * 1.0

        time += delta / 1000.00

        jump = jumpAnimation.duration
        beforeJump = 1.0
        blendIn = 0.4
        blendOut = 0.4
        blendOutStart = beforeJump + jump - blendOut
        total = 3.75

        root = skeleton.bones[0]
        speed = 180.0

        if time > beforeJump + blendIn and time < blendOutStart:
            speed = 360.0

        root.x = root.x + speed * delta / 1000.00
        
        screen.fill((0, 0, 0))

        if time > total:
            # restart
            time = 0.0
            root.x = -50.0
        elif time > beforeJump + jump:
            # just walk after jump
            walkAnimation.apply(skeleton, time, True)
        elif time > blendOutStart:
            # blend out jump
            walkAnimation.apply(skeleton, time, True)
            jumpAnimation.mix(skeleton, time - beforeJump, False, 1.0 - (time - blendOutStart) / blendOut)
        elif time > beforeJump + blendIn:
            # just jump
            jumpAnimation.apply(skeleton, time - beforeJump, False)
        elif time > beforeJump:
            # blend in jump
            walkAnimation.apply(skeleton, time, True)
            jumpAnimation.mix(skeleton, time - beforeJump, False, (time - beforeJump) / blendIn)
        else:
            # just walk before jump
            walkAnimation.apply(skeleton, time, True)

        skeleton.updateWorldTransform()
        skeleton.update(clock.get_time())
        skeleton.draw(screen, 0)
        pygame.display.set_caption('%s  %.2f' % (caption, clock.get_fps()), 'Spine Runtime')
        pygame.display.flip()
pygame.quit()
